#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     trs2Text.awk                                                    #
# Authors:  Brigitte Bigi                                                   #
# Date:     march, 2007                                                     #
# Licence:  GPL                                                             #
# Brief:    Get speech turns of the ESTER corpus                            #
#                                                                           #
# ######################################################################### #
 


BEGIN{
	# Booleen : dans un tour de parole ?
	inTurn = 0
	# Booleen : tour de parole contenant ... du texte ?
	TurnOK = 0

	# Ecriture de l'entete xml
	print "\n<corpus>\n"
}

/<Trans/{
	# Extraction du nom de fichier audio
	if (i=match($0, /audio_filename=/))
	{
		tmp=substr($0,i+16,length($0))
		docref=substr(tmp,0,match(tmp,/"/)-1)
	}
}

# Extrait le nom d'un nouveau thme
# ---------------------------------
/<Topic[^s]/{
	# Extraction d'une ligne du type :
	# <Topic id="to1" desc="Autrement dit "/>
	# On cre un tableau du type attribut/valeur avec :
	# - la 1ere chaine entre guillements est l'attribut
	# - la 2nde chaine entre guillemets est la valeur 
	nb = split($0, a, "\"")
	topic[a[2]] = a[4]
}

# Dbut d'un tour de parole
# -------------------------
/<Turn/{
	if (inTurn == 1)
	{
		print "Bad xml file format with <Turn>" > "/dev/stderr"
		exit 1
	}
	inTurn = 1
}



# Fin d'un tour de parole
# -----------------------
/<\/Turn/{
	if (inTurn == 0)
	{
		print "Bad xml file format with </Turn>" > "/dev/stderr"
		exit 1
	}
	if (TurnOK == 1)
		print "\n<><>"
	inTurn = 0
	TurnOK = 0
}



# Dbut d'un "document"
# ---------------------
/<Section/{
	if (inTurn == 1)
	{
		print "Bad xml file format with <Section>" > "/dev/stderr"
		exit 1
	}
	if (length(docref)==0)
	{
		print "<doc>"
	}
	else
	{
		printf "<doc ref=%s>\n", docref
	}

	# Extraction du code du thme (si appropri)
	if (match($0, /topic=/) != 0)
	{
		nb = split($0, a, "\"")
		print "<descr>"
		print "<title>"
		print topic[a[4]]
		print "</title>"
		print "</descr>"
	}
	print "<text>\n<><>"
}


# Fin d'un "document"
# -------------------
/<\/Section/{
	if (inTurn == 1)
	{
		print "Bad xml file format with </Section>" > "/dev/stderr"
		exit 1
	}
	print "</text>"
	print "</doc>"
}

# Autres cas
# ----------
(inTurn == 1) {
        # Supprime les redondances d'espaces et espaces en debut de chaine
        # idem pour les tabulations...
        gsub("[  ]+", " ")
        gsub(/^ /, "")
        gsub(/^	/, "")

	if (!match($0, /^</) && length($0) > 0)
	{
		gsub (/ \. /, " . <>\n")
		printf "%s ", $0
		TurnOK = 1
	}
}

