#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Html2Text.awk                                                   #
# Authors:  Brigitte Bigi                                                   #
# Date:     april, 2006                                                     #
# Licence:  GPL                                                             #
# Brief:    HTML filter                                                     #
# Help:     Html2Text.awk -h                                                #
#                                                                           #
# Input:    Only ONE html file                                              #
# Output:   The text with:                                                  #
#              - an XML-like header with (if any):                          #
#                  - keywords                                               #
#                  - title                                                  #
#              - a semi-structured text where:                              #
#                  - <><> means a new paragraph                             #
#                  - <>   means a new sentence                              #
#                                                                           #
# Caution: Even if comments or scripts are only a part of line, the         #
#          entire line will be ignored.                                     #
# ######################################################################### #



# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Html2Text.awk [options]\n\n"
        printf "Where options include:\n"
        printf "  -h    print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
                        FCT_Usage()
                }
        }
}
# ######################################################################### #
#                                                                           #
# Function Name: FCT_Replace                                                #
#                                                                           #
# ######################################################################### #
function FCT_Replace(str, start, nb)
{
	# On cherche le " de terminaison, SAUF
	# s'il n'y a pas de " juste apres le =, on cherche le prochain espace
	j=match(substr(str,start+nb-1,1),/"/)
	if (j==0)
	{
		j=match(substr(str,start+nb,length(str)),/ /)
		k=match(substr(str,start+nb,length(str)),/>/)
		if ((k!=0) && (k<j)) j=k-1
	}
	else
		j=match(substr(str,start+nb,length(str)),/"/)
	
	if (j==0)
	{
		# si on ne trouve pas de " pour terminer l'attribut,
		# on cherche la fin du tag
		j=match(substr(str,start+nb,length(str)),/>/)
		if (j==0)
		{
			# aucune terminaison... ca veut dire que
			# le tag ou l'attribut est sur plusieurs lignes
			# ou malformation !
			j=length(str)-start-nb
			print "###j="j > "/dev/stderr"
		}
		j=j-1
	}
	_W1=substr(str,0,start-1)
	_W2=substr(str,start+j+nb,length(str))

	return _W1 _W2
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_RemoveAttributes                                       #
#                                                                           #
# ######################################################################### #
function FCT_RemoveAttributes ( str )
{
	# Supprime les redondances d'espaces et espaces en debut de chaine
	# idem pour les tabulations...
	#gsub("&amp", " ", str)
	gsub("&nbsp;", " ", str)
	gsub("[  ]+", " ", str)
	gsub("^ ", "", str)
	gsub("^ ", "", str)

	if (i=match(str,/[cC][eE][lL][lL][pP][aA][dD][dD][iI][nN][gG]=/))	#cellpadding
		str=FCT_Replace(str,i,13)
	if (i=match(str,/[cC][eE][lL][lL][sS][pP][aA][cC][iI][nN][gG]=/))	#cellspacing
		str=FCT_Replace(str,i,13)
	if (i=match(str,/[oO][nN][mM][oO][uU][sS][eE][oO][vV][eE][rR]=/))	#onmouseover
		str=FCT_Replace(str,i,13)
	if (i=match(str,/[oO][nN][mM][oO][uU][sS][eE][oO][uU][tT]=/))		#onmouseout
		str=FCT_Replace(str,i,12)
	if (i=match(str,/[aA][cC][cC][eE][sS][sS][kK][eE][yY]=/))		#accesskey
		str=FCT_Replace(str,i,11)
	if (i=match(str,/[cC][oO][lL][sS][pP][aA][nN]=/))			#colspan
		str=FCT_Replace(str,i,9)
	if (i=match(str,/[rR][oO][wW][sS][pP][aA][nN]=/))			#rowspan
		str=FCT_Replace(str,i,9)
	if (i=match(str,/[oO][nN][fF][oO][cC][uU][sS]=/))			#onfocus
		str=FCT_Replace(str,i,9)
	if (i=match(str,/[oO][nN][cC][lL][iI][cC][kK]=/))			#onclick
		str=FCT_Replace(str,i,9)
	if (i=match(str,/[bB][gG][cC][oO][lL][oO][rR]=/))			#bgcolor
		str=FCT_Replace(str,i,9)
	if (i=match(str,/[dD][iI][sS][pP][lL][aA][yY]=/))			#display
		str=FCT_Replace(str,i,9)
	if (i=match(str,/[aA][cC][tT][iI][oO][nN]=/))				#action
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[mM][eE][tT][hH][oO][dD]=/))				#method
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[vV][aA][lL][iI][gG][nN]=/))				#valign
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[bB][oO][rR][dD][eE][rR]=/))				#border
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[hH][eE][iI][gG][hH][tT]=/))				#height
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[uU][sS][eR][mM][aA][pP]=/))				#usermap
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[cC][uU][sS][tT][oO][mM]=/))				#custom
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[cC][oO][oO][rR][dD][sS]=/))				#coords
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[tT][aA][rR][gG][eE][tT]=/))				#target
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[hH][sP][pP][aA][cC][eE]=/))				#hspace
		str=FCT_Replace(str,i,8)
	if (i=match(str,/[tT][iI][tT][lL][eE]=/))				#title
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[wW][iI][dD][tT][hH]=/))				#width
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[aA][lL][iI][gG][nN]=/))				#align
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[cC][oO][lL][oO][rR]=/))				#color
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[sS][tT][yY][lL][eE]=/))				#style
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[sS][hH][aA][pP][eE]=/))				#shape
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[sS][tT][aA][tT][eE]=/))				#state
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[cC][lL][aA][sS][sS]=/))				#class
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[vV][aA][lL][uU][eE]=/))				#value
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[lL][aA][bB][eE][lL]=/))				#label
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[iI][nN][Pp][uU][tT]=/))				#input
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[tT][bB][oO][dD][yY]=/))				#tbody
		str=FCT_Replace(str,i,7)
	if (i=match(str,/[hH][rR][eE][fF]=/))					#href
		str=FCT_Replace(str,i,6)
	if (i=match(str,/[fF][aA][cC][eE]=/))					#face
		str=FCT_Replace(str,i,6)
	if (i=match(str,/[nN][aA][mM][eE]=/))					#name
		str=FCT_Replace(str,i,6)
	if (i=match(str,/[Tt][yY][pP][eE]=/))					#type
		str=FCT_Replace(str,i,6)
	if (i=match(str,/[sS][iI][zZ][eE]=/))					#size
		str=FCT_Replace(str,i,6)
	if (i=match(str,/[sS][pP][aA][nN]=/))					#span
		str=FCT_Replace(str,i,6)
	if (i=match(str,/[lL][aA][nN][gG]=/))					#lang
		str=FCT_Replace(str,i,6)
	if (i=match(str,/[xX][mM][lL]=/))					#xml
		str=FCT_Replace(str,i,5)
	if (i=match(str,/[rR][eE][lL]=/))					#rel
		str=FCT_Replace(str,i,5)
	if (i=match(str,/[aA][lL][tT]=/))					#alt
		str=FCT_Replace(str,i,5)
	if (i=match(str,/[sS][rR][cC]=/))					#src
		str=FCT_Replace(str,i,5)
	if (i=match(str,/[rR][eE][lL]=/))					#rel
		str=FCT_Replace(str,i,5)
	if (i=match(str,/[iI][dD]=/))						#id
		str=FCT_Replace(str,i,4)
	if (i=match(str,/[tT][dD]=/))						#td
		str=FCT_Replace(str,i,4)
	if (i=match(str,/[tT][rR]=/))						#tr
		str=FCT_Replace(str,i,4)

	gsub(/\\=""/, "", str)
	gsub(/\"\"/, "", str)
	gsub("[  ]+", " ", str)
	gsub(/ >/, ">", str)

	return str
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_RemoveTags                                             #
#                                                                           #
# ######################################################################### #
function FCT_RemoveTags ( str )
{
	# Pour la mise en forme, on recupere les tags significatifs
	# du document d'origine et on les remplace par :
	#	- le symbole <> pour les phrases
	#	- le symbole <><> pour les paragraphes
	gsub(/\r/, " <> ", str)			# ^M
	gsub("<[Bb][Rr]>", " <> ", str)		# <br>
	gsub("<[Bb][Rr] />", " <> ", str)	# <br />
	gsub("<[/]?[Pp]>", " <><> ", str)	# <p> </p>
	gsub("<[/]?[Ll][Ii]>", " <> ", str)	# <li> </li>
	gsub("<[/]?[Tt][Rr]>", " <> ", str)	# <tr> </tr>
	gsub("<[/]?[Tt][Dd]>", " <> ", str)	# <td> </td>
	gsub("</[Dd][Ii][Vv]>", " <><> ", str)	# <div> </div>
	gsub("</[Hh][1-9]+>", " <> ", str)	# <h?> </h?>

	# On supprime tous les tags
	gsub("<[/]?[a-zA-Z0-9: ]+[ ]?[/]?>", " ", str)	# tag entier
	gsub("<[/]?[a-zA-Z0-9: ]+[ ]?[/]? <>", " ", str)	# tag sur plusieurs lignes

	# Gestion des espaces et tabulations
	gsub("\t", " ", str)
	gsub("[  ]+", " ", str)
	gsub("^ ", "", str)

	# Attention, certains peuvent etre mal forms!!!
	# on les supprime maintenant :
	do {
		cont=0
		l=length(str)
		i=match(str,"<")
		j=match(str,">")
		if (j>(i+1))
		{
			cont=1
			_W1 = substr(str,0,i-1)
			_W2 = substr(str,j+1,length(str))
			str = _W1 _W2
		}
	}
	while (cont==1)

	# Gestion des espaces et tabulations
	gsub("\t", " ", str)
	gsub("[  ]+", " ", str)
	gsub("^ ", "", str)

	return str
}

# ############################# BEGIN MAIN ################################ #

BEGIN {
        FCT_CommandLine(ARGC, ARGV)
        delete ARGV
        ARGC=1

	# Ecriture de l'entete... 
	print "\n<corpus>"
	print "\n<doc>"

	inhead    = 0	# Booleen : dans l'entete ?
	inbody    = 0	# Booleen : dans le document ?
	inscript  = 0	# Booleen : dans un script ?
	incomment = 0	# Booleen : dans un commentaire ?
}

/<head/ || /<HEAD/ {
	inhead = 1
	print "<descr>"
}

/<\/head/ || /<HEAD/ {
	inhead = 0
	print "</descr>"
}

/<body/ || /<BODY/ {
	if (inhead == 1)
	{
		print "</descr>"
		inhead = 0	# au cas ou il manque le </head>
	}
	print "\n<text>"
	print "\n<><>"
	inbody    = 1	# Debut du contenu du document
	inpar     = 1	# Booleen : dans un paragraphe ?
	insent    = 0	# Booleen : dans une phrase ?
	next
}

/<script/ || /<SCRIPT/ {
	inscript = 1
}

/<\/script/ || /<\/SCRIPT/ {
	inscript = 0
	next
}

/<!--/ {
	incomment = 1
}

/-->/ {	
	incomment = 0
	next
}


/meta name=/ && (inhead == 1) {
	if (match($0,/name="title"/))
	{
	        # Get the title content 
		c = substr($0, index($0, "content"), length($0))
	        s = index(c, "\"")				# start: 1st " after content
	        e = index(substr(c, s+1, length(c)), "\"")	# end: 1st " after start
	        w = substr(c, s+1, e-1)				# word from start to end
		print "<title>"
		print w
		print "</title>"
	}
	if (match($0,/name="keywords"/))
	{
	        # Get the title content 
		c = substr($0, index($0, "content"), length($0))
	        s = index(c, "\"")				# start: 1st " after content
	        e = index(substr(c, s+1, length(c)), "\"")	# end: 1st " after start
	        w = substr(c, s+1, e-1)				# key words from start to end
		gsub(/, /, " </k>\n<k> ", w)			# <k> mot-cle </k>
		gsub(/,/, " </k>\n<k> ",  w)
		print "<keys>"
		print "<k>", w, "</k>"
		print "</keys>"
	}
}

(inbody == 1 && inscript == 0 && incomment == 0) {

	# Supprime les attributs puis les tags
	do {
		l = length($0)
		$0 = FCT_RemoveAttributes($0)
	}
	while (length($0)!=l)

	do {
		l = length($0)
		$0 = FCT_RemoveTags($0)
	}
	while (length($0)!=l)

	# Cette ligne commence t-elle par un symbole de paragraphe <><> 
	if (match($0,/^[ ]*<><>/))
		newpar = 1
	else
		newpar = 0

	# Segmente la ligne en paragraphes
	nbpar = split($0, TAB_Par, "<><>")

	# Traite separement chaque paragraphe
	for (par=1; par<=nbpar; par++)
	{
		# Ecriture du symbole de nouveau paragraphe (si approprie)
		if ( inpar == 0 )
		{
			if ( ( par == 1 && newpar == 1 ) || par > 1 )
			{
				if (insent == 0)
					print "<><>"
				else
					print "<>\n<><>"
				inpar = 1
				insent = 0
			}
		}

		# Verifie qu'il y a un contenu dans le paragraphe
		line = TAB_Par[par] 
		gsub(/<>[ ]*/, "", line)
		if (length(line) > 0)
		{
			# Remplace les \n par le symbole de phrase <>
			gsub(/\n/, " <> ", TAB_Par[par])

			# Cette ligne finie t-elle par un symbole de phrase <>
			endsent = 0
			if (match(TAB_Par[par],/<>[ ]*$/)) endsent = 1

			# On decoupe la ligne en phrases
			nbsent = split(TAB_Par[par], TAB_Sent, "<>")

			# Traite separement chaque phrase
			for (sent=1; sent<nbsent; sent++)
			{
				str = TAB_Sent[sent]
				gsub("[  ]+", " ", str)
				gsub("^ ", "", str)
				if (length(str) > 0)
				{
					printf "%s <>\n", str
					inpar = 0
				}
				insent = 0
			}
			# Derniere phrase (speciale car peut etre tronquee)
			str = TAB_Sent[nbsent]
			gsub("[  ]+", " ", str)
			gsub("^ ", "", str)
			if (length(str) > 0)
			{
				if (endsent == 1)
				{
					# Phrase terminee correctement
					printf "%s <>\n", str
					insent = 0
				}
				else
				{
					# Phrase probablement tronquee
					printf "%s ", str
					insent = 1
				}
				inpar  = 0
			}
		}
	}
}

END {
	print "\n</text>"
	print "\n</doc>"
	print "\n</corpus>"
}

################################ END MAIN ###################################


