#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Num2LetterKH.awk                                                #
# Authors:  Brigitte Bigi     & Sopheap SENG                                #
# Date:     oct, 2007                                                       #
# Licence:  GPL                                                             #
# Brief:    Convert numbers of a corpus to the corresponding Khmer string   #
# Help:     Num2LetterKH.awk -h                                             #
#                                                                           #
# ######################################################################### #



# ######################################################################### #
#                                                                           #
# Function: unite                                                           #
# 	Converts numbers from 0 to 9 and returns the corresponding string   #
#                                                                           #
# ######################################################################### #
function unite( number )
{
	_r = ""
	if (number == 9)  _r = " ប្រាំបួន "
	if (number == 8)  _r = " ប្រាំបី "
	if (number == 7)  _r = " ប្រាំពីរ "
	if (number == 6)  _r = " ប្រាំមួយ "
	if (number == 5)  _r = " ប្រាំ "
	if (number == 4)  _r = " បួន "
	if (number == 3)  _r = " បី "
	if (number == 2)  _r = " ពីរ "
	if (number == 1)  _r = " មួយ "
	if (number == 0)
	{
		_str = " (ស្សូន) "
		for(i=1; i<length(number); i++)
			_str = " " _str "- ស្សូន "
		return _str
	}
	return _r
}

# ######################################################################### #
#                                                                           #
# Function: dizaine                                                         #
# 	Converts numbers from 1 to 99 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function dizaine( number )
{
	if (number >= 90 && number <= 99)
	{
		_str = "កៅសិប "
		if (number > 90)
			_str = _str unite(number-90)

	}
	else if (number >= 80 && number <= 89)
	{
		_str = "ប៉ែតសិប "
		if (number > 80)
			_str = _str unite(number-80)
	}
	else if (number >= 70 && number <= 79)
	{
		_str = "ចិតសិប "
		if (number > 70)
			_str = _str unite(number-70)

	}
	else if (number >= 60 && number <= 69)
	{
		_str = "ហុកសិប "
		if (number > 60)
			_str = _str unite(number-60)
	}
	else if (number >= 50 && number <= 59)
	{
		_str = "ហាសិប "
		if (number > 50)
			_str = _str unite(number-50)
	}
	else if (number >= 40 && number <= 49)
	{
		_str = "សែសិប "
		if (number > 40)
			_str = _str unite(number-40)
	}
	else if (number >= 30 && number <= 39)
	{
		_str = "សាមសិប "
		if (number > 30)
			_str = _str unite(number-30)
	}
	else if (number >= 20 && number <= 29)
	{
		_str = "ម្ភៃ "
		if (number > 20)
			_str = _str unite(number-20)

	}
	else if (number >= 10 && number <= 19)
	{
		_str = "ដប់ "
		if (number > 10)
			_str = _str unite(number-10)
	}
	else
		_str = unite(number)

	return _str
}	


# ######################################################################### #
#                                                                           #
# Function: centaine                                                        #
# 	Converts numbers from 1 to 999 and returns the corresponding string #
#                                                                           #
# ######################################################################### #
function centaine( number )
{
	if (number >= 100)
	{
		if (number == 900)
			_str =  "ប្រាំបួន រយ " 
		else if (number > 900 && number <= 999)
			_str =  "ប្រាំបួន រយ " dizaine(number-900)
		else if (number == 800)
			_str =  "ប្រាំបី រយ " 
		else if (number > 800 && number <= 899)
			_str =  "ប្រាំបី រយ " dizaine(number-800)
		else if (number == 700)
			_str =  "ប្រាំពីរ រយ " 
		else if (number > 700 && number <= 799)
			_str =  "ប្រាំពីរ រយ " dizaine(number-700)
		else if (number == 600)
			_str =  "ប្រាំមួយ រយ " 
		else if (number > 600 && number <= 699)
			_str =  "ប្រាំមួយ រយ " dizaine(number-600)
		else if (number == 500)
			_str =  "ប្រាំ រយ " 
		else if (number > 500 && number <= 599)
			_str =  "ប្រាំ រយ " dizaine(number-500)
		else if (number == 400)
			_str =  "បួន រយ " 
		else if (number > 400 && number <= 499)
			_str =  "បួន រយ " dizaine(number-400)
		else if (number == 300)
			_str =  "បី រយ " 
		else if (number > 300 && number <= 399)
			_str =  "បី រយ " dizaine(number-300)
		else if (number == 200)
			_str =  "ពីរ រយ " 
		else if (number > 200 && number <= 299)
			_str =  "ពីរ រយ " dizaine(number-200)
		else if (number > 100 && number <= 199)
			_str =  "មួយ រយ " dizaine(number-100)
		if (number == 100)
			_str =  "មួយ រយ " 
	}
	else
		_str = dizaine(number)
		
	return _str	
}	


# ######################################################################### #
#                                                                           #
# Function: millier                                                         #
# Converts numbers from 1000 to 99999 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function millier( number )
{
	if (number < 1000) _str = centaine(number)

	# Milliers
	# --------

	if ( ( number >= 1000 ) && ( number < 10000 ) )
	{
		if ( (number%1000) != 0 )
			_str = unite(int (number/1000)) " ពាន់ " centaine(number%1000)
		else
			_str = unite(int (number/1000)) " ពាន់ "
	}
	return _str
}
# ######################################################################### #
#                                                                           #
# Function: millier                                                         #
# Converts numbers from 1000 to 99999 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function dizainesMilliers( number )
{
	# Dizaines de milliers
	# --------------------

	if (number < 10000) _str = millier(number)

	if (number == 10000)
		_str = "មួយ ម៉ឺន "
	
	if (number > 10000 && number < 100000)
	{
		if ( (number%10000) != 0 )
			_str = unite(int (number/10000)) " ម៉ឺន " millier(number%10000)
		else
			_str = unite(int (number/10000)) " ម៉ឺន "
	}
	return _str
}

# ######################################################################### #
#                                                                           #
# Function: Centaines de milliers                                           #
# Converts numbers from 1000 to 99999 and returns the corresponding string  #
#                                                                           #
# ######################################################################### #
function centainesMilliers( number )
{	
	if (number < 100000) _str = dizainesMilliers(number)

	# Centaines de milliers
	# ---------------------
	if (number == 100000)
		_str = "មួយ សែន"
	
	if (number > 100000 && number < 1000000)
	{
		if ( (number%100000) != 0 )
			_str = unite(int (number/100000)) " សែន " dizainesMilliers(int (number%100000))
		else
			_str = unite(int (number/100000)) " សែន "
	}

	return _str
}


# ######################################################################### #
#                                                                           #
# Function: FCT_Num2text                                                    #
#	Converts all numbers and returns the corresponding string           #
#                                                                           #
#  number is the number to convert                                          #
#                                                                           #
# ######################################################################### #
function FCT_Num2text( number )
{

	if (number < 1000000)
		_str = centainesMilliers(number)


	# លាន
	# --------
	
	if (number == 1000000)
		_str = "មួយ លាន"
	
	if (number > 1000000 && number < 1000000000)
	{
		if ( (number%1000000) != 0 )
			_str = centaine(int (number/1000000)) " លាន " centainesMilliers(int (number%1000000))
		else
			_str = centaine(int (number/1000000)) " លាន "
	}


	if (number > 1000000000)
		return number

	return _str
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Num2LetterKH.awk [options]\n\n"
        printf "Where options include:\n"
	printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -noxml       input is one word a line (instead of XML)\n"
        printf "  -h	print  this help\n"
	printf "\n"
	printf " input number: stdin\n"
	printf " output text:  stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] == "-noxml")
                {
                        xml = 0
                        i++
                }
                else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
                else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
        }
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Convert                                                #
#                                                                           #
# Description:                                                              #
#     Get a number and return a string                                      #
#                                                                           #
# ######################################################################### #

function FCT_Convert ( w )
{

	# If input string w is Khmer number, 
	# convert it to Latin number in order to compute

	_w = FCT_ConvertKHnum2Latin(w)
	_strnum = ""

	# Numbers starting by one or more "0"
	if (match(_w,/^0/))
	{
		while (match(_w,/^0/))
		{
			_strnum = _strnum " ស្សូន "
			sub(/^0/, "", _w)
		}
	}
	gsub(/\./, "",_w)
	if (length(_w) > 0)
		_strnum = _strnum FCT_Num2text(int(_w))
	return _strnum
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_ConvertKHnum2Latin                                     #
#                                                                           #
# Description:                                                              #
#     Convert Khmer number to Latin                                         #
#                                                                           #
# ######################################################################### #

function FCT_ConvertKHnum2Latin( w )
{

	_strnum = w

	gsub(/០/, "0", _strnum)
	gsub(/១/, "1", _strnum)
	gsub(/២/, "2", _strnum)
	gsub(/៣/, "3", _strnum)
	gsub(/៤/, "4", _strnum)
	gsub(/៥/, "5", _strnum)
	gsub(/៦/, "6", _strnum)
	gsub(/៧/, "7", _strnum)
	gsub(/៨/, "8", _strnum)
	gsub(/៩/, "9", _strnum)

	return _strnum
}


# ############################# BEGIN MAIN ################################ #

BEGIN {
	Output      = "/dev/stdout"	# output file name
	xml         = 1			# xml input ??? (yes is default)
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1
}

/<str>/ {
        # Get the element
        s = index($0, ">")				# start: 1st >
        e = index(substr($0, s+1, length($0)), "<")	# end: 1st < after start
        w = substr($0, s+1, e-1)			# word: $0 from start to end
        gsub(/ /, "", w)				# remove spaces

	# Verify if the input string is a number : 
	# Latin number 01234... or Khmer Number ០១២៣៤
	nombre = w
	sub(/([0-9]*)|([០១២៣៤៥៦៧៨៩]*)/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert(w)

		# Format the output (see dtd!)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		gsub(" $", "", strnum)		# remove an ending space
		gsub(" ", " </str>\n</wd>\n<wd>\n<str> ", strnum)	# add tags
		strnum = "<str> " strnum " </str>"			# add tags

		# Print
		print strnum > Output
		next
	}
}

(xml == 0) {
	# Verify if the input string is a number
	nombre = $0
	sub(/([0-9]*)|([០១២៣៤៥៦៧៨៩]*)/, "", nombre)

	# Num2Text only numbers! 
	if (length(nombre) == 0)
	{
		strnum = FCT_Convert($0)
		gsub("[  ]+", " ", strnum)	# remove multiple spaces
		gsub("^ ", "", strnum)		# remove a starting space
		print strnum > Output
		next
	}
}

{
	print > Output
}

# ############################## END MAIN ################################# #


