#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# Program Name:  Text2XML.awk                                               #
# Author:        Brigitte Bigi                                              #
# Date:          April, 2006                                                #
# Licence:       GPL                                                        #
# Brief:         Add XML tags to a semi-structured text                     #
# Help:          Text2XML.awk -h                                            #
#                                                                           #
# Input: a text where:                                                      #
#       - a <><> means a new paragraph                                      #
#       - a <>   means a new sentence                                       #
# Modified by Viet Bac LE (09/2007): adding of an UTF-8 convertor           #
#									    #
# ######################################################################### #


# ######################################################################### #
#                                                                           #
# Function Name: _ord_init(), ord(c), chr(c)                                #
#                                                                           #
# Description:                                                              #
#     Translating Between Characters and Numbers                            #
#                                                                           #
# Global identifiers:							    #
#    _ord_:        numerical values indexed by characters                   #
#    _ord_init:    function to initialize _ord_                             #
#                                                                           #
# ######################################################################### #
			    
function _ord_init()
{
	low = 0
	high = 255
	
	for (i = low; i <= high; i++) {
		t = sprintf("%c", i)
		_ord_[t] = i
	}
}

function ord(c)
{
	# only first character is of interest
	#c = substr(str, 1, 1)
	return _ord_[c]
}

function chr(c)					            
{
	# force c to be numeric by adding 0
	return sprintf("%c", c + 0)
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_NumtoUTF8                             		    #
#                                                                           #
# Description:                                                              #
#     Convert a number to utf8 character                                    #
#                                                                           #
# ######################################################################### #
function FCT_NumtoUTF8 ( num )
{
	if (int(num) < 128) {
		num_utf = sprintf("%c", num)
	}
	else {			
		if (int(num) < 2048) {				
			num1 = or(192, rshift(int(num), 6))
			num2 = or(128, and(int(num), 63))
			num_utf = sprintf("%c%c", num1, num2)					
		}
		else {
			if (int(num) < 65536) {
				num1 = or(224, rshift(int(num), 12))
				num2 = or(128, and(rshift(int(num), 6), 63))
				num3 = or(128, and(int(num), 63))
				num_utf = sprintf("%c%c%c", num1, num2, num3)
			}
			else num_utf = ""
		}
	}
	
	return num_utf
}

# ######################################################################### #
#                                                                           #
# Function Name: FCT_UCStoUTF8                             		    #
#                                                                           #
# Description:                                                              #
#     Convert a HTML Unicode Entity Ref (&#xxxx;) to utf8 character         #
#                                                                           #
# ######################################################################### #
function FCT_UCStoUTF8 ( str )
{
	do {
		i = match(str, /&#([0-9]+);/, matched)
		if (i > 0) {
			l = length(matched[0])
			num = matched[1]						
			num_utf = FCT_NumtoUTF8(num) 
			str = substr(str, 1, i-1) num_utf substr(str, i+l)
		}
	}
	while (i>0)
	
	return str
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_FromHTML                             		    #
#                                                                           #
# Description:                                                              #
#     Normalize text extracted from a HTML document                         #
#                                                                           #
# ######################################################################### #
function FCT_FromHTML ( str )
{
	gsub(/&iexcl;/, "", str)
	gsub(/&cent;/, "", str)
	gsub(/&pound;/, "", str)
	gsub(/&curren;/, "", str)
	gsub(/&yen;/, "", str)
	gsub(/&brvbar;/, "", str)
	gsub(/&sect;/, "", str)
	gsub(/&uml;/, "", str)
	gsub(/&copy;/, "  ", str)
	gsub(/&ordf;/, "", str)
	gsub(/&laquo;/, "", str)
	gsub(/&not;/, "", str)
	gsub(/&shy;/, "", str)
	gsub(/&reg;/, "  ", str)
	gsub(/&macr;/, "", str)
	gsub(/&deg;/, "", str)
	gsub(/&plusmn;/, "", str)
	gsub(/&sup2;/, "", str)
	gsub(/&sup3;/, "", str)
	gsub(/&acute;/, "", str)
	gsub(/&micro;/, "", str)
	gsub(/&para;/, "", str)
	gsub(/&middot;/, "", str)
	gsub(/&cedil;/, "", str)
	gsub(/&sup1;/, "", str)
	gsub(/&ordm;/, "", str)
	gsub(/&raquo;/, "", str)
	gsub(/&frac14;/, "", str)
	gsub(/&frac12;/, "", str)
	gsub(/&frac34;/, "", str)
	gsub(/&iquest;/, "", str)
	gsub(/&Agrave;/, "", str)
	gsub(/&Aacute;/, "", str)
	gsub(/&aacute;/, "", str)
	gsub(/&Acirc;/, "", str)
	gsub(/&Atilde;/, "", str)
	gsub(/&Auml;/, "", str)
	gsub(/&Aring;/, "", str)
	gsub(/&AElig;/, "", str)
	gsub(/&Ccedil;/, "", str)
	gsub(/&Egrave;/, "", str)
	gsub(/&Eacute;/, "", str)
	gsub(/&Ecirc;/, "", str)
	gsub(/&Euml;/, "", str)
	gsub(/&Igrave;/, "", str)
	gsub(/&Iacute;/, "", str)
	gsub(/&Icirc;/, "", str)
	gsub(/&Iuml;/, "", str)
	gsub(/&ETH;/, "", str)
	gsub(/&Ntilde;/, "", str)
	gsub(/&Ograve;/, "", str)
	gsub(/&Oacute;/, "", str)
	gsub(/&oacute;/, "", str)
	gsub(/&Ocirc;/, "", str)
	gsub(/&Otilde;/, "", str)
	gsub(/&Ouml;/, "", str)
	gsub(/&times;/, "", str)
	gsub(/&Oslash;/, "", str)
	gsub(/&Ugrave;/, "", str)
	gsub(/&Uacute;/, "", str)
	gsub(/&Ucirc;/, "", str)
	gsub(/&Uuml;/, "", str)
	gsub(/&Yacute;/, "", str)
	gsub(/&THORN;/, "", str)
	gsub(/&szlig;/, "", str)
	gsub(/&agrave;/, "", str)
	gsub(/&aacute;/, "", str)
	gsub(/&acirc;/, "", str)
	gsub(/&atilde;/, "", str)
	gsub(/&auml;/, "", str)
	gsub(/&aring;/, "", str)
	gsub(/&aelig;/, "", str)
	gsub(/&ccedil;/, "", str)
	gsub(/&egrave;/, "", str)
	gsub(/&eacute;/, "", str)
	gsub(/&ecirc;/, "", str)
	gsub(/&euml;/, "", str)
	gsub(/&igrave;/, "", str)
	gsub(/&iacute;/, "", str)
	gsub(/&icirc;/, "", str)
	gsub(/&iuml;/, "", str)
	gsub(/&eth;/, "", str)
	gsub(/&ntilde;/, "", str)
	gsub(/&ograve;/, "", str)
	gsub(/&oacute;/, "", str)
	gsub(/&ocirc;/, "", str)
	gsub(/&otilde;/, "", str)
	gsub(/&ouml;/, "", str)
	gsub(/&divide;/, "", str)
	gsub(/&oslash;/, "", str)
	gsub(/&ugrave;/, "", str)
	gsub(/&uacute;/, "", str)
	gsub(/&ucirc;/, "", str)
	gsub(/&uuml;/, "", str)
	gsub(/&yacute;/, "", str)
	gsub(/&thorn;/, "", str)
	gsub(/&yuml;/, "", str)
	gsub(/&quot;/, "\"", str)
	gsub(/&lt;/, "<", str)
	gsub(/&gt;/, ">", str)
	gsub(/&#8211;/, "-", str) 
	gsub(/&apos;/, "'", str)
	gsub(/&#8217;/, "'", str)
	gsub(/&#8201;/, "_", str)
	gsub(/&#8364;/, " euros ", str)
	gsub(/&euro;/, " euros ", str)
	gsub(/&#x2013;/, "-", str)
 	gsub(/&#160;/, " ", str)
 	gsub(/&bull;/, ".", str)
 	gsub(/&#8230;/, "...", str)

	if (!match(str,/\&amp/))
		gsub(/&/, "_\\&amp;_", str)

	return str
}



# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Text2XML.awk [options]\n\n"
        printf "Where options include:\n"
        printf "  -o filename  write the output to this file (default: stdout)\n"
	printf "  -u           convert to utf-8\n"
	printf "  -e enc       specify output encoding (default is utf-8)\n"
	printf "  -l LANG      give a specific language: FR|CN (default: none)\n"
        printf "  -h           print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout\n"
        exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
	utf8encoding = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
		if (argv[i] == "-u")
			utf8encoding = 1
		else
                if (argv[i] == "-o")
                {
                        Output = argv[i+1]
                        i++
                }
		else
                if (argv[i] == "-e")
                {
                        encname = argv[i+1]
                        i++
                }
                else
                if (argv[i] == "-l")
                {
                        lang = argv[i+1]
                        i++
                }
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
                        FCT_Usage()
                }
        }
}


# ############################# BEGIN MAIN ################################ #


BEGIN {
        Output      = "/dev/stdout"     # output file name
	encname     = "utf8"		# default output encoding
	lang        = "none"		# default language
        FCT_CommandLine(ARGC, ARGV)	# extract parameters
        delete ARGV
        ARGC=1

        intext = 0       # in the text ?
        inpar = 0        # in a paragraph ?
	_ord_init()	 # run once this initialization

	printf "<?xml version=\"1.0\" encoding=\"%s\"?>\n", encname > Output
	print "<!DOCTYPE corpus SYSTEM \"corpus.dtd\">" > Output
}

/<text>/ {
        intext = 1
        print > Output
        next
}

/<\/text>/ {
        intext = 0
        if (inpar == 1)
                print "\n</orig>\n</sent>\n</par>\n" > Output
        print > Output
        inpar = 0
        next
}


(intext == 1) {
        # Make some cleaning!
        gsub("[ ]+", " ")
        gsub("^ ", "")
	gsub("<[/]?[[:alnum:]=\" ]+[/]?>", " ")
	gsub("<[/]?[[:alnum:]=\" ]+[/]?", " ")

	# Accents and encoding
	$0 = FCT_FromHTML($0)
	if (utf8encoding == 1) 
	{
		l = length($0)
		str = ""
		for (i = 1; i <= l; i++) {
			c = substr($0, i, 1)
			num = _ord_[c]
			if (int(num) < 128) str = str c
			else str = str FCT_NumtoUTF8(num)
		}
		$0 = str
		$0 = FCT_UCStoUTF8($0)
	}

        # New paragraph?
        if (match($0, /^<><>/))
        {
                if (inpar == 1)
                {
                        print "\n</orig>\n</sent>\n</par>\n" > Output
                }
                print "<par>\n<sent>\n<orig>\n" > Output
                inpar = 1
                next
        }

        if (inpar == 1 && length($0) > 0)
        {
                # Buffer 
                line = ""

                # String analysis (separated by spaces)
                for (i=1; i<=NF; i++)
                {
		  if (lang != "XX")
                  {
			# Segmente en phrase au "." si : 
                        #  - un mot en minuscule ou un chiffre est colle avant le point
                        #    sauf si un mot est colle apres le point (acronymes, adresses...)
                        #    sauf si la lettre en question est c.
                        #  - si c'est une majuscule, attention a la longueur (3 caracteres) 
                        #    pour ne pas couper les M. (monsieur), par exemple
                        #    sauf si un mot est colle apres le point (acronymes, adresses...)
                        #  - un point seul, avec des espaces autour
			if (lang=="FR")
			{
                        	if ((!match($i, /\.[[:alnum:]"]/)))
				{
	                                $i = gensub(/([a-z0-9%")])\./, "\\1.\n</orig>\n</sent>\n<sent>\n<orig>\n", "g", $i)
                                	if (length($i) > 3)
                                        	$i = gensub(/([A-Z])\./, "\\1.\n</orig>\n</sent>\n<sent>\n<orig>\n", "g", $i)
				}
                        }
			else
                                $i = gensub(/([[:alnum:]")])\./, "\\1.\n</orig>\n</sent>\n<sent>\n<orig>\n", "g", $i)

                        if (length($i)==1 && $i==".")
                                sub(/\./, ".\n</orig>\n</sent>\n<sent>\n<orig>\n", $i)                   

			# En chinois simplifi, une fin de phrase est explicite avec un caractre special utf-8
			if (lang == "CN")
			{
				str=gensub(/(.)/, "\\\\\\1", "g", orig)
				cmd = ("echo " str " | $CTTK/bin/SplitSentCN.py")
				cmd | getline orig
				close(cmd)
			}

		####################################
		#### 22 juin 2010
                        # Replace "?"
			#gsub(/? [A-Z]/, "?\n</orig>\n</sent>\n<sent>\n<orig>\n", $i)
                        # Replace "!"
                        #gsub(/! [A-Z]/, "!\n</orig>\n</sent>\n<sent>\n<orig>\n", $i)
		####################################
                  }
			
                  # Replace "<>"
                  gsub(/<>$/, "\n</orig>\n</sent>\n<sent>\n<orig>\n", $i)

                  line = line $i "_"
                }
                gsub(/[_]+/, " ", line)

                # Replace "..." by "... TAGS" only if an upper case follows
                if (lang != "XX")
		{
		####################################
			line = gensub(/![ ]+([A-Z])/, "!\n</orig>\n</sent>\n<sent>\n<orig>\n\\1", "g", line)
			line = gensub(/?[ ]+([A-Z])/, "?\n</orig>\n</sent>\n<sent>\n<orig>\n\\1", "g", line)
		####################################
                	line = gensub(/\.\.\.[ ]+([A-Z])/, " ...\n</orig>\n</sent>\n<sent>\n<orig>\n\\1", "g", line)
                }

                # and voila...
                printf "%s\n", line > Output
        }
        else
                print > Output
}

(intext == 0) {
        print > Output
}

################################ END MAIN ###################################


