#!/bin/tcsh
# ######################################################################### #
#                                                                           #
# File:    HTML2XML.csh                                                     #
# Author:  Brigitte Bigi                                                    #
# Date:    May, 2007                                                        #
# Licence: GPL                                                              #
#                                                                           #
# Description:                                                              #
#	Convert a HTML document to an XML (see corpus.dtd)                  #
#                                                                           #
# Parameters:                                                               #
#       $1 = HTML file or URL                                               #
#       $2 = -w|-g                                                          #
#       $3 = -w|-g                                                          #
#                                                                           #
# Output:                                                                   #
#       a XML file on stdout                                                #
#                                                                           #
# ######################################################################### #


# ######################################################################### #
# Command line extraction
# ######################################################################### #
if ($#argv == 0) then
	set filename=`zenity --title "HTML2XML - Select a file to convert" --file-selection`
	set graphical="true"
endif

# Verify the number of arguments
# ------------------------------
if ($#argv == 1 && $1 == "-h") then
        # Bip, print usage and exit
        printf '\a'
        echo "Usage: " `basename $0` " [HTML|URL] [w] [g] [-h]" > "/dev/stderr"
        exit 1
endif

if ($#argv > 0) then
	# Set the graphical mode
	# ----------------------
	set graphical="false"
        if ( $2 == "g" || $3 == "g") then
                if (-X zenity) then
                        set graphical="true"
                else
                        echo "Warning: Graphical mode is impossible on your system"
                endif
	endif
        if ( $2 == "w" || $3 == "w") then
		# Get the HTML file on the web
		# ----------------------------
		wget --quiet $1
		set filename=`basename $1`
	else
		set filename=$1
	endif
endif

# Verify the HTML file
# --------------------
if (! -e $filename) then
        if ($graphical == "false") then
       	        echo "ERROR: Cant find the file " $filename  > "/dev/stderr"
        else
       	        zenity --title "HTML2XML - ERROR" --error --text "\nCant find the file $1\n" 100 100
        endif
       	exit 1
endif

# Verify the toolkit is ok
# ------------------------
set tk=`env | grep -c CTTK`
if ($tk == "0") then
        if ($graphical == "false") then
       	        echo "ERROR: the environment variable CTTK is not set"  > "/dev/stderr"
        else
       	        zenity --title "HTML2XML - ERROR" --error --text "\nthe environment variable CTTK is not set\n" 100 100
        endif
       	exit 1
endif
set liatk=`env | grep -c LIA_TAGG=`

# ######################################################################### #
# OK, now start to really work!!!
# ######################################################################### #

if ($graphical == "true") then
	if (-e progress) rm progress
	mkfifo progress
	zenity --title "HTML2XML - Progress" --progress --text "Please wait while processing the file..." --percentage 0 --auto-close < progress > progress &
else
	echo -n "[                   ]\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
endif

set file = `basename $filename .html`

# Document level
# --------------
cat $filename | $CTTK/bin/HTMLClean.csh | $CTTK/bin/Html2Text.awk  | $CTTK/bin/Text2XML.awk -l FR -e iso-8859-1 > $file.xml.1

if ($graphical == "true") then
	echo 10 > progress
else
	echo -n "#"
endif

# Element level
# --------------
cat $file.xml.1 | $CTTK/bin/SplitOrig2Wd.awk  | \
                  $CTTK/bin/Replace.awk -V $CTTK/dico/Replace-fr.dico -l FR > $file.xml.2
if ($graphical == "true") then
	echo 20 > progress
else
	echo -n "##"
endif

# Word level 
# --------------
cat $file.xml.2 | $CTTK/bin/TokenizeFR.awk -V $CTTK/dico/Lexique-fr.vocab > $file.xml.3
if ($graphical == "true") then
	echo 30 > progress
else
	echo -n "##"
endif

cat $file.xml.3 | $CTTK/bin/Stick.awk    -V $CTTK/dico/Phrases-fr.vocab -a "_" -s 7 > $file.xml.5
if ($graphical == "true") then
	echo 50 > progress
else
	echo -n "####"
endif

cat $file.xml.5 | $CTTK/bin/Num2LetterFR.awk > $file.xml.6
if ($graphical == "true") then
	echo 80 > progress
else
	echo -n "##"
endif

if ($liatk == "0") then
	cat $file.xml.6 | $CTTK/bin/Lower.awk -V $CTTK/dico/UpperLower-fr.dico > $file.xml.7
else
	cat $file.xml.6 | $CTTK/bin/PosTagger.awk -l | $CTTK/bin/Lower.awk -V $CTTK/dico/UpperLower-fr.dico > $file.xml.7
endif
if ($graphical == "true") then
	echo 85 > progress
else
	echo -n "##"
endif

cat $file.xml.7 | $CTTK/bin/Remove.awk   -V $CTTK/dico/Punct.vocab -w > $file.xml
if ($graphical == "true") then
	echo 99 > progress
else
	echo -n "##"
endif

# Clean
rm $file.xml.?
if ($graphical == "true") then
	echo 100 > progress
else
	echo " ]"
endif
if ($graphical == "true") rm progress

