#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# Program Name:  Counts2Vocab.awk                                           #
# Author:        Brigitte Bigi                                              #
# Date:          May, 2004                                                  #
# Licence:       GPL                                                        #
# Brief:         Select the vocabulary words from a unigram count file      #
#                                                                           #
# ######################################################################### #



# ######################################################################### #
#                                                                           #
#  Function Name: FCT_LoadCounts                                            #
#                                                                           #
#  Description:                                                             #
#    Load the word/count from a unigram count file                          #
#                                                                           #
#  Parameter:                                                               #
#    filename is a string                                                   #
#                                                                           #
# ######################################################################### #
function FCT_LoadCounts ( filename )
{
	_ind = 0
	while ( getline < filename )
	{
		if (NF != 2)
		{
			printf "Error in the file %s, bad number of columns\n", filename > "/dev/stderr"
			exit 1
		}
		_word  = $1
		_count = $2
		if ( _count > MinCount )
		{
			TAB_Words[$1] = $2
			_ind++
		}
	}
	return _ind
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
	printf "Usage: Counts2Vocab.awk [options]\n\n"
	printf "Where options include:\n"
	printf "  -t filename    read unigram counts from this file\n"
	printf "  -o filename    write sorted vocabulary to this file\n"
	printf "  -K value       desired vocabulary size\n"
	printf "  -c value       desired minimum number of occurrences for a word\n"
	printf "  -h	         print this help\n"
	exit
}


# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
	entry = 0
	for (i=1; i<argc; i++)
	{
		if (argv[i] == "-h")
			FCT_Usage()
		else
		if (argv[i] == "-o")
		{
			OutputVocab = argv[i+1]
			i++
		}
		else
		if (argv[i] == "-t")
		{
			InputWFreq=argv[i+1]
			i++
		}
		else
		if (argv[i] == "-c")
		{
			MinCount=argv[i+1]
			i++
		}
		else
	 	if (argv[i] == "-K")
		{
			VocabSize=argv[i+1]
			i++
		}
		else
		if (argv[i] ~ /^\-/)
		{
			printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
			FCT_Usage()
		}
		else
			break
	}
}


# ############################# BEGIN MAIN ################################ #

BEGIN {
	# Init parameters
	VocabSize   = 0			# vocab size (0 means everything)
	OutputVocab = "/dev/stdout"	# vocab file name
	InputWFreq  = "NULL"		# wfreq file name
	MinCount    = 0			# minimum value for a count

	# Extract parameters from command line
	if ( ARGC == 1 ) FCT_Usage()	# need help...
	FCT_CommandLine(ARGC, ARGV)     # extract true parameters
	delete ARGV
	ARGC=1

	if (InputWFreq == "NULL")
	{
		print "Error: You need to define an input filename" > "/dev/stderr"
		FCT_Usage()
	}

	# Load word/counts
	FCT_LoadCounts(InputWFreq)

	# Define the unix command which select the vocab
	if (VocabSize == 0)
	{
		command = sprintf ("cut -f1 | sort > %s", OutputVocab)
	}
	else
	{
		command = sprintf ("sort -k 1 -nr | head -n %s | cut -f1 | sort > %s", VocabSize,OutputVocab)
	}

	# Set the vocab
	for (word in TAB_Words)
		printf "%s\t%d\n", word, TAB_Words[word] | command 
	close(command)

	exit
}
 
################################ END MAIN ###################################
 

