#! /usr/bin/gawk -f
# ######################################################################### #
#                                                                           #
# File:     Giza2XML.awk                                                    #
# Authors:  Brigitte Bigi                                                   #
# Date:     sept, 2007                                                      #
# Licence:  GPL                                                             #
# Brief:    Translation format converter                                    #
# Help:     Giza2XML.awk -h                                                 #
#                                                                           #
# Input:    a Giza++ translation file                                       #
# Output:   a XML file (following corpus.dtd)                               #
#                                                                           #
# ######################################################################### #

# ######################################################################### #
#                                                                           #
# Function Name: FCT_Usage                                                  #
#                                                                           #
# Description:                                                              #
#     Print the usage and exit the program                                  #
#                                                                           #
# ######################################################################### #
function FCT_Usage ()
{
        printf "Usage: Giza2XML.awk [options]\n\n"
        printf "Where options include:\n"
        printf "  -h    print this help\n"
        printf "\n"
        printf " input text:  stdin\n"
        printf " output text: stdout\n"
        exit
}



# ######################################################################### #
#                                                                           #
# Function Name: FCT_CommandLine                                            #
#                                                                           #
# Description:                                                              #
#     Get arguments from the command line                                   #
#                                                                           #
# ######################################################################### #
function FCT_CommandLine(argc, argv)
{
        entry = 0
        for (i=1; i<argc; i++)
        {
                if (argv[i] == "-h")
                        FCT_Usage()
                else
                if (argv[i] ~ /^\-/)
                {
                        printf "invalid option -- %s\n", argv[i] > "/dev/stderr"
                        FCT_Usage()
                }
                        break
        }
}

# ############################# BEGIN MAIN ################################ #

BEGIN {
        FCT_CommandLine(ARGC, ARGV)
        delete ARGV
        ARGC=1

        # Ecriture de l'entete xml... 
        # ##### A Noter que l'encodage devrait etre lu dans le charset 
        # ##### du document d'origine.
        print "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>"
        print "<!DOCTYPE corpus SYSTEM \"corpus.dtd\">"
        print "\n<corpus>"
        print "\n<doc>"
        print "\n<descr>"
        print "\n</descr>"
        print "\n<text>"
}

# New sentence pair
/^#/{
	# INFORMATIONS
	# Make some cleaning
	gsub(/\(/, "")
	gsub(/\)/, "")
	# Get informations from this line:
	# 1. get sentence number: $4
	# 2. get source length: $7
	# 3. get target length: $10
	# 4. get alignment score: $NF
	numsent=$4
	srclength=$7
	tgtlength=$10
	score=$NF
	
	# TARGET SENTENCE
	getline
	if (NF!=tgtlength) {print "ERROR target length, sentence number",nbsent;exit}
	# Split the sentence
	split($0,TAB_Tgtsent, " ")

	# SOURCE SENTENCE
	getline
	# Split the sentence and get alignments
	w=0	# new word boolean false
	a=0	# new alignment boolean false
	align=0	# alignment tab index
	src=0	# words tab index
	for (i=2; i<=NF; i++)
	{
		if (match($i, /\({/))
		{
			w=0
			a=1
		}
		else if (match($i, /}\)/))
		{
			# end alignment
			w=1
			a=0
		}
		else
		{
			if (w==1)
			# new word
			{
				src++			# inc. index
				TAB_Srcsent[src]=$i	# get word
			}
			else if (a==1)
			# new alignment
			{
				align++			# inc. index
				TAB_AlignSrc[align]=src	# get src num
				TAB_AlignTgt[align]=$i	# get tgt num
			}
		}
	}
	if (src!=srclength) {print "ERROR source length, sentence number",nbsent;exit}
	
	# OK, now print XML sentence
	# --------------------------
	printf "<sent num=\"%d\">\n",numsent 

	# Print source
	printf "<elts id=\"source\" length=\"%s\">\n", srclength
	for (i=1; i<=srclength; i++)
		printf "<wd num=\"%d\"> %s </wd>\n", i, TAB_Srcsent[i] 
	print "</elts>"

	# Print target
	printf "<elts id=\"target\" length=\"%s\">\n", tgtlength
	for (i=1; i<=tgtlength; i++)
		printf "<wd num=\"%d\"> %s </wd>\n", i, TAB_Tgtsent[i] 
	print "</elts>"

	# Print alignments
	printf "<align idsrc=\"source\" idtgt=\"target\" score=\"%s\">\n", score
	for (i=1; i<=align; i++)
		printf "<a numsrc=\"%d\" numtgt=\"%d\" />\n", TAB_AlignSrc[i],TAB_AlignTgt[i]
	print "</align>"

	print "</sent>"
}

END {
        print "\n</text>"
        print "\n</doc>"
        print "\n</corpus>"
}

################################ END MAIN ###################################



